; $Id: dumpgen.pro,v 1.1 1998/07/27 13:30:49 rgr Exp $

; $RCSfile: dumpgen.pro,v $
; $Author: rgr $
; $Date: 1998/07/27 13:30:49 $
; $Revision: 1.1 $

; This procedure is used to write an IDL procedure which will do a
; formatted dump of selected fields of a list of data structures.
; The fields and format are controlled by a template file which
; contains the name of the dump procedure, a list of the type of
; structures in the argument list, a list of the argument names,
; the dump template, and a dictionary of segments to be replaced
; in the template.  Optionally, the template file can redefine the
; delimiters for marking segments to be replaced in the template,
; delimiters for marking macro values to be substituted into the
; segment variables, and the symbol used to mark macros in the 
; segment variables.
; 
; Arguments
;   infile = name of input template file-(input)
;   outfile = name of file to contain generated procedure-(input)
;
; Keywords
;   rcs = Flag for embedding RCS commands in the generated procedure
;
;
; Procedures and functions
;   buildSegmentDictionary
;   FieldBreak
;   findDumpLine
;   genDumpLine


pro dumpGen, infile, outfile, RCS=rcs

   ; Copy the contents of the template input file into the string
   ; array called line.

   line = strarr(1024)
   get_lun, lun
   openr, lun, infile
   nline = 0
   tmp = ""
   while (not eof(lun)) do begin
      readf, lun, tmp
      line[nline] = tmp
      nline = nline + 1
   endwhile
   close, lun
   free_lun, lun

   ; Get the delimiters for the segments to be replaced in the
   ; template.  The default segment delimiters are "<" and ">".

   segmentDelimiter= ["<", ">"]
   index = findDumpLine(line, "!SegmentDelimiters:")
   if (index ge 0) then begin
      FieldBreak, line[index], fields, nfields
      if (nfields eq 2) then begin
         segmentDelimiter[0] = strmid(fields[1], 0, 1)
         segmentDelimiter[1] = strmid(fields[1], 1, 1)
      endif else begin
         stop, "Error in segment delimiters."
      endelse
   endif

   ; Get the delimiters for the macro values to be substituted into
   ; the segment variables.  The default macro value delimiters are
   ; "{" and "}".

   macroDelimiter= ["{", "}"]
   index = findDumpLine(line, "!MacroDelimiters:")
   if (index ge 0) then begin
      FieldBreak, line[index], fields, nfields
      if (nfields eq 2) then begin
         macroDelimiter[0] = strmid(fields[1], 0, 1)
         macroDelimiter[1] = strmid(fields[1], 1, 1)
      endif else begin
         stop, "Error in macro delimiters."
      endelse
   endif

   ; Get the symbol for marking macros in the segemnt variables.  The
   ; default macro symbol is "$".

   macroSymbol = "$"
   index = findDumpLine(line, "!MacroSymbol:")
   if (index ge 0) then begin
      FieldBreak, line[index], fields, nfields
      if (nfields eq 2) then begin
         macroSymbol = fields[1]
      endif else begin
         stop, "Error in macro symbol."
      endelse
   endif

   ; Get the name of the generated dump procedure from the information
   ; that was in the input template file.

   index = findDumpLine(line, "!Procedure:")
   if (index ge 0) then begin
      FieldBreak, line[index], fields, nfields
      if (nfields eq 2) then begin
         ProcedureName = fields[1]
      endif else begin
         stop, "Cannot locate procedure name."
      endelse
   endif else begin
      stop, "Cannot locate procedure name."
   endelse

   ; Get the names of the arguments (ignore the types) which will be
   ; used in the generated dump procedure.

   index = findDumpLine(line, "!ArgumentList:")
   if (index ge 0) then begin
      FieldBreak, line[index], fields, nfields
      if (nfields ge 2) then begin
         arg = where ((strpos(fields, ",") ge 0), narg)
         if (narg gt 0) then                            $
            arg = [fields[arg], fields[nfields - 1]]    $
         else                                           $
            arg = fields[nfields - 1]
         narg = narg + 1
      endif else begin
         stop, "Cannot locate argument list."
      endelse
   endif else begin
      stop, "Cannot locate argument list."
   endelse

   ; Find the beginning and ending of the dictionary of the segments
   ; to be replaced in the template and build the segment dictionary.

   index0 = findDumpLine(line, "!BeginDictionary") + 1
   index1 = findDumpLine(line, "!EndDictionary") - 1
   if ((index0 le 0) or (index0 gt index1)) then begin
      stop, "Cannot find dictionary."
   endif
   buildSegmentDictionary, line[index0:index1], segmentDelimiter,     $
                           macroDelimiter, dictName, dictVariable,    $
                           dictFormat, dictNameCompressed

   ; Find the beginning and ending of the dump template.

   index0 = findDumpLine(line, "!BeginTemplate") + 1
   index1 = findDumpLine(line, "!EndTemplate") - 1
   if ((index0 le 0) or (index0 gt index1)) then begin
      stop, "Cannot find template."
   endif

   ; Open the output file which will contain the generated dump
   ; procedure.

   get_lun, lun
   openw, lun, outfile

   ; Generate the RCS commands for the dump procedure.

   if (keyword_set(rcs)) then begin
      printf, lun, "; $Id: dumpgen.pro,v 1.1 1998/07/27 13:30:49 rgr Exp $"
      printf, lun
      printf, lun, "; $RCSfile: dumpgen.pro,v $"
      printf, lun, "; $Author: rgr $"
      printf, lun, "; $Date: 1998/07/27 13:30:49 $"
      printf, lun, "; $Revision: 1.1 $"
      printf, lun
      printf, lun
   endif

   ; Generate the procedure line for the dump procedure.

   code = "pro " + ProcedureName + ","
   for i=0, narg-1 do               $
      code = code + " " + arg[i]
   printf, lun, code, ", LUN=lun"
   printf, lun

   ; Generate the code that makes the printer the default device. but
   ; However, if the keyword LUN is set, then the output is assigned
   ; to the device associated with lun.

   printf, lun, "   lunit = -1"
   printf, lun, "   if (keyword_set(lun)) then lunit = lun"
   printf, lun


   ; Embedded the dump template into the dump procedure.

   tmp = strcompress(string((index1 - index0) + 1), /remove_all)
   printf, lun, "   template = strarr(" + tmp + ")"
   printf, lun
   ndigit = strlen(strcompress(string(index1 - index0), /remove_all))
   width = strcompress(string(ndigit), /remove_all)
   fmt = "(i" + width + "." + width + ")"
   ntemplate = 0
   for i=index0, index1 do begin
      tmp = string(ntemplate, format=fmt)
      printf, lun, '   template[' + tmp + '] = "' + line[i] + '"'
      ntemplate = ntemplate + 1
   endfor
   printf, lun

   ; Generate the code to fill in each line of the dump template.

   ntemplate = 0
   for i=index0, index1 do begin
      genDumpLine, lun, ntemplate, line[i], segmentDelimiter,   $
                   macroDelimiter, macroSymbol, dictName,       $
                   dictVariable, dictFormat, dictNameCompressed
      ntemplate = ntemplate + 1
   endfor

   ; Write the end statement for the generated procedure.

   printf, lun
   printf, lun, "end"

   ; Close the file that contains the generated procedure.

   close, lun
   free_lun, lun

end
